CVRPLIB_URL = "http://vrp.atd-lab.inf.puc-rio.br/media/com_vrp/instances/"


CVRP_SETS = [
    "A",
    "B",
    "E",
    "F",
    "M",
    "P",
    "CMT",
    "tai",
    "Golden",
    "Li",
    "X",
    "XXL",
]

VRPTW_SETS = [
    "Solomon",
    "HG",
]

XXL_NAMES = ["Antwerp", "Brussels", "Flanders", "Ghent", "Leuven"]
DIMACS_NAMES = ["Loggi", "ORTEC"]
